package org.codehaus.xfire.aegis;

import java.io.InputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.configuration.PlexusConfigurationException;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.Initializable;
import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.aegis.operation.WrappedOperation;
import org.codehaus.xfire.aegis.wsdl.AegisWSDL;
import org.codehaus.xfire.fault.SOAP11FaultHandler;
import org.codehaus.xfire.fault.SOAP12FaultHandler;
import org.codehaus.xfire.handler.Handler;
import org.codehaus.xfire.handler.dom.DOMHandler;
import org.codehaus.xfire.handler.dom.DOMPipelineHandler;
import org.codehaus.xfire.plexus.simple.PlexusSimpleService;
import org.codehaus.xfire.wsdl.WSDLBuilder;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.QName;
import org.dom4j.io.SAXReader;

/**
 * AegisService
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class AegisService
    extends PlexusSimpleService
    implements Initializable
{
    private Document serviceDescriptor;

    private Map operations;
    
    private String serviceClassName;
    
    private TypeRegistry typeRegistry;
    
    private PlexusConfiguration[] handlers;
    
    private Handler serviceHandler;
    
    public AegisService()
    {
    	operations = new HashMap();
    }

	/**
	 * @see org.apache.avalon.framework.activity.Initializable#initialize()
	 */
	public void initialize() throws Exception
	{
        super.initialize();
        
        DOMPipelineHandler pipe = (DOMPipelineHandler) getServiceHandler();

        if ( handlers != null && handlers.length > 0 )
        {
            for ( int i = 0; i < handlers.length; i++ )
            {
                String hId = handlers[i].getValue();
                
                DOMHandler handler = (DOMHandler) getServiceLocator().lookup( DOMHandler.ROLE, hId );
                
                pipe.getHandlers().add( handler );
            }
        }
        else
        {
            pipe.getHandlers().add( new AegisServiceHandler() );
        }
        
        typeRegistry = (TypeRegistry) getServiceLocator().lookup(TypeRegistry.ROLE);
        setWSDLBuilder((WSDLBuilder) getServiceLocator().lookup(WSDLBuilder.ROLE, "aegis"));
        
		SAXReader xmlReader = new SAXReader();
        serviceDescriptor = xmlReader.read( getServiceDescriptorStream() );

        createService();

        AegisWSDL wsdl = (AegisWSDL) getWSDL();
	}

    /**
     * Create a service and operations from the service descriptor.
     */
	protected void createService()
	{
		Element root = serviceDescriptor.getRootElement();
        setDefaultNamespace( root.attribute("targetNamespace").getStringValue() );
        setSoapVersion( root.attribute("soapVersion").getStringValue() );
        
        if ( getSoapVersion().equals ( SOAPConstants.SOAP12_ENVELOPE_NS ))
        {
        	setFaultHandlerHint( SOAP12FaultHandler.NAME );
        }
        else
        {
            setFaultHandlerHint( SOAP11FaultHandler.NAME );
        }
        
        List elements = root.elements("operation");
        for ( Iterator itr = elements.iterator(); itr.hasNext(); )
        {
        	Element opEl = (Element) itr.next();
            
            WrappedOperation op = new WrappedOperation();
            op.configure( opEl, this, getTypeRegistry() );
            
            QName opQ = op.getQName();

            operations.put( opQ, op );
        }
	}

	public Collection getOperations()
	{
		return operations.values();
	}
    
    public WrappedOperation getOperation( QName name )
    {
        return (WrappedOperation) operations.get( name );
    }
    
	public TypeRegistry getTypeRegistry()
	{
		return typeRegistry;
	}

	/**
     * Returns an InputStream for the descriptor.
     * @return InputStream representing the descriptor
     */
    public InputStream getServiceDescriptorStream()
    {
        String name = getServiceDescriptorName();
        InputStream is = getClass().getResourceAsStream( name );
        
        if ( is == null )
            throw new XFireRuntimeException("No service descriptor found: " + name);
        
        return is;
    }

    public String getServiceDescriptorName()
    {
        String full = getServiceClassName();
        full = "/" + full.replace('.', '/');

        return full + ".xml";
    }
    
	public Document getServiceDescriptor()
	{
		return serviceDescriptor;
	}
    
	public String getServiceClassName()
	{
		return serviceClassName;
	}
    
	public void setServiceClassName(String serviceClassName)
	{
		this.serviceClassName = serviceClassName;
	}
    
    public Object createServiceObject()
    {
        if ( getServiceLocator().hasComponent(getServiceClassName()) )
        {
        	try
			{
				return getServiceLocator().lookup( getServiceClassName() );
			}
			catch (ComponentLookupException e)
			{
				throw new XFireRuntimeException("Couldn't lookup service.", e);
			}
        }
        else
        {
        	ClassLoader cl = Thread.currentThread().getContextClassLoader();
            
            Class clazz;
			try
			{
				clazz = cl.loadClass(getServiceClassName());
                return clazz.newInstance();
			}
			catch (Exception e)
			{
				throw new XFireRuntimeException("Couldn't create service class.", e);
			}
			
        }
    }
    
    /**
     * @see org.apache.avalon.framework.configuration.Configurable#configure(org.apache.avalon.framework.configuration.Configuration)
     */
    public void configure( PlexusConfiguration config ) throws PlexusConfigurationException
    {
        super.configure( config );
        
        setFaultHandlerHint( config.getChild("faultHandler").getValue("aegis") );
        
        setServiceClassName( config.getChild("serviceClass").getValue() );
        setStyle( SOAPConstants.STYLE_WRAPPED );
        setUse( SOAPConstants.USE_LITERAL );
        
        DOMPipelineHandler handler = new DOMPipelineHandler();
        setServiceHandler( handler );
        
        handlers = config.getChild("handlers").getChildren("handler");
    }
    
    
    public Handler getServiceHandler()
    {
        return serviceHandler;
    }
    
    public void setServiceHandler( Handler serviceHandler )
    {
        this.serviceHandler = serviceHandler;
    }
}
