package org.codehaus.xfire.aegis;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;
import org.codehaus.xfire.MessageContext;
import org.codehaus.xfire.aegis.operation.WrappedOperation;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.handler.dom.AbstractDOMHandler;
import org.codehaus.xfire.handler.dom.DOMHandler;
import org.codehaus.xfire.handler.dom.Message;
import org.dom4j.Element;

/**
 * AegisServiceProvider
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class AegisServiceHandler
	extends AbstractDOMHandler
    implements DOMHandler
{
	public static final String NAME = "aegis";
    
    /**
     * @see org.codehaus.xfire.handler.dom.DOMHandler#invoke(org.codehaus.xfire.MessageContext, org.codehaus.xfire.handler.dom.Message, org.codehaus.xfire.handler.dom.Message)
     */
    public void invoke( MessageContext context, 
                        Message request, 
                        Message response ) 
        throws Exception
    {
		AegisService service = (AegisService) context.getService();
        
        Object serviceOb = service.createServiceObject();

        WrappedOperation op = service.getOperation( getPayload(request).getQName() );
        
        if ( op == null )
        {
        	throw new XFireFault("No such operation.", XFireFault.SENDER);
        }
        
        Map mapContext = new HashMap();
        
        op.read( request, mapContext);
        
        Map responseVars = null;
        try
		{
            Method m = serviceOb.getClass().getMethod( op.getQName().getName(), new Class[] { Map.class } );
            
			responseVars =  (Map) m.invoke( serviceOb, new Object[] { mapContext } );
		}
        catch( InvocationTargetException e )
        {
            // ITEs are such a pain.
            Throwable t = e.getTargetException();
        	if ( t instanceof Exception )
            {
                if ( t instanceof XFireFault )
                    throw (XFireFault) t;
                
        		throw new XFireFault("Error invoking method.",
                                     (Exception) t,
                                     XFireFault.RECEIVER);
            }
            
            throw e;
        }
        
        op.write(response, responseVars);
	}

	public Element getPayload( Message request )
    {
        return (Element) request.getBody().elements().get(0);
    } 
}
