package org.codehaus.xfire.aegis.mapping;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.naming.ConfigurationException;

import org.codehaus.plexus.configuration.PlexusConfiguration;
import org.codehaus.plexus.configuration.PlexusConfigurationException;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.Configurable;
import org.codehaus.xfire.AbstractXFireComponent;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.type.Type;
import org.dom4j.QName;

/**
 * DefaultTypeRegistry
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class DefaultTypeRegistry
	extends AbstractXFireComponent
	implements TypeRegistry, Configurable
{
	private Map types = new HashMap();
    
	/**
	 * @see org.codehaus.xfire.aegis.mapping.TypeRegistry#register(org.dom4j.QName, java.lang.Class)
	 */
	public void register(QName typeQ, Class typeClass)
	{
		types.put( typeQ, typeClass );
	}

	/**
	 * @see org.codehaus.xfire.aegis.mapping.TypeRegistry#getTypeClass(org.dom4j.QName)
	 */
	public Class getTypeClass(QName typeQ)
	{
		return (Class) types.get(typeQ);
	}
    
	/**
	 * @see org.codehaus.xfire.aegis.mapping.TypeRegistry#createType(org.dom4j.QName)
	 */
	public Type createType(QName typeQ)
	{
		try
		{
            Class clazz = getTypeClass(typeQ);
            
            if ( clazz == null )
                return null;
            
            Type type = (Type) clazz.newInstance();
            
			return type;
		}
		catch (InstantiationException e)
		{
			throw new XFireRuntimeException("Could not create type.", e);
		}
		catch (IllegalAccessException e)
		{
			throw new XFireRuntimeException("Could not create type.", e);
		}
	}

	/**
	 * @see org.codehaus.xfire.aegis.mapping.TypeRegistry#unregisterType(org.dom4j.QName)
	 */
	public Class unregisterType(QName typeQ)
	{
		return (Class) types.remove( typeQ );
	}

	/**
	 * @see org.codehaus.xfire.aegis.mapping.TypeRegistry#getAllTypes()
	 */
	public Collection getAllTypes()
	{
		return types.values();
	}

	/**
	 * @see org.apache.avalon.framework.configuration.Configurable#configure(org.apache.avalon.framework.configuration.Configuration)
	 */
	public void configure(PlexusConfiguration config) 
        throws PlexusConfigurationException
	{
		PlexusConfiguration[] types = config.getChildren("types");
        
        for ( int i = 0; i < types.length; i++ )
        {
            String namespace = types[i].getAttribute("namespace");
            
            PlexusConfiguration[] type = types[i].getChildren("type");
            
            for ( int j = 0; j < type.length; j++ )
            {
            	createType( type[j], namespace );
            }
        }
	}

	/**
	 * @param configuration
	 * @param namespace
	 * @throws ConfigurationException
	 */
	private void createType(PlexusConfiguration configuration, String namespace)
        throws PlexusConfigurationException
	{
		String name = configuration.getAttribute("name");
        String className = configuration.getAttribute("type");
        
        ClassLoader cl = Thread.currentThread().getContextClassLoader();
        
        QName q = QName.get(name, namespace);
        try
		{
			register(q, cl.loadClass(className));
		}
		catch (ClassNotFoundException e)
		{
			throw new PlexusConfigurationException("Couldn't configure type.", e);
		}
	}
}
