package org.codehaus.xfire.aegis.operation;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.aegis.type.Type;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.handler.dom.Message;
import org.dom4j.Element;
import org.dom4j.QName;

/**
 * WrappedOperation
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class WrappedOperation
{
    private List request;
    private List response;
    private List requestHeader;
    private List responseHeader;
    private QName qName;
    private String documentation;
    
    public WrappedOperation()
    {
    	request = new ArrayList();
        response = new ArrayList();
        requestHeader = new ArrayList();
        responseHeader = new ArrayList();
    }
    
    public void write( Message message, Map vars )
        throws XFireFault
    {
        for ( Iterator itr = responseHeader.iterator(); itr.hasNext(); )
        {
            Type type = (Type) itr.next();
            type.write(message.getHeader(), vars);
        }
        
        QName responseQ = QName.get( getQName().getName() + "Response", getQName().getNamespace() );
        
        Element responseEl = message.getBody().addElement(responseQ);
        
        for ( Iterator itr = response.iterator(); itr.hasNext(); )
        {
            Type type = (Type) itr.next();
            type.write(responseEl, vars);
        }
    }

    public void read( Message message, Map vars )
        throws XFireFault
    {
        Element header = message.getHeader();
        if ( header != null )
        {
            for ( Iterator itr = requestHeader.iterator(); itr.hasNext(); )
            {
                Type type = (Type) itr.next();
                type.read(message.getHeader(), vars);
            }
        }
        
        Element payload = (Element) message.getBody().elements().get(0);
        
        for ( Iterator itr = request.iterator(); itr.hasNext(); )
        {
            Type type = (Type) itr.next();
            type.read(payload, vars);
        }
    }

    public void configure( Element config, AegisService service, TypeRegistry reg )
    {
        setQName( QName.get(config.attributeValue("name"), service.getDefaultNamespace()) );
        setDocumentation( config.getTextTrim() );

        configureTypes( service, response, reg, config.element("response") );
        configureTypes( service, request, reg, config.element("request") );
        configureTypes( service, responseHeader, reg, config.element("responseHeader") );
        configureTypes( service, requestHeader, reg, config.element("requestHeader") );
    }
    

	private void configureTypes(AegisService service, List types, TypeRegistry reg, Element resEl)
	{
        if ( resEl == null )
            return;
        
		List resElements = resEl.elements();
        for ( Iterator itr = resElements.iterator(); itr.hasNext(); )
        {
            Element paramEl = (Element) itr.next();
            Type type = createType(service, reg, paramEl);
            
            types.add(type);
        }
	}

	/**
	 * @param service
	 * @param reg
	 * @param paramEl
	 * @return
	 */
	private Type createType(AegisService service, TypeRegistry reg, Element paramEl)
	{
		QName typeQ = QName.get( paramEl.getName(), service.getSoapVersion() );
		
		Type type = reg.createType(typeQ);
        
        if ( type == null )
            throw new XFireRuntimeException( "No such type exists: " + typeQ );
        
		type.configure(paramEl, service, reg);
		return type;
	}

	public List getRequest()
	{
		return request;
	}
    
	public void setRequest(List request)
	{
		this.request = request;
	}
    
	public List getResponse()
	{
		return response;
	}
    
	public void setResponse(List response)
	{
		this.response = response;
	}

	public QName getQName()
	{
		return qName;
	}
    
    public void setQName(QName name)
    {
        qName = name;
    }
    
	public String getDocumentation()
	{
		return documentation;
	}
    
	public void setDocumentation(String documentation)
	{
		this.documentation = documentation;
	}
}
