package org.codehaus.xfire.aegis.type;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ognl.Ognl;
import ognl.OgnlException;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.util.NamespaceHelper;
import org.dom4j.Attribute;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse </a>
 */
public class ArrayType
	extends Type
{
	private String key;
    
    private String arrayClass;
    
    private Type childType;
    
    private String postOgnl;
    
    /**
     * Uses an OGNL expression to find an array.  It then puts each
     * array object in as the key (see <code>getKey()</code> for retrieval
     * when write()'ing in the child type.
     * 
     * @see org.codehaus.xfire.aegis.type.Type#write(org.dom4j.Element, java.util.Map)
     */
	public void write(Element element, Map context) throws XFireFault
	{
		try
        {   
            Object value = null;
            try
            {
                value =  Ognl.getValue(getOgnl(), context, (Object) null);
            }
            catch ( NullPointerException e )
            {
                return;
            }
            
            if ( value == null )
            {
            	return;
            }
            
            Element typeEl = element.addElement(getQName());

            if ( value instanceof Object[] )
            {
                Object[] children = (Object[]) value;
                
                for (int i = 0; i < children.length; i++)
                {
                    context.put( getKey(), children[i] );
                    
                    Type type = getChildType();
                    type.write(typeEl, context);
                }
            }
            else if ( value instanceof Collection )
            {
                Collection children = (Collection) value;
                for ( Iterator itr = children.iterator(); itr.hasNext(); )
                {
                    context.put( getKey(), itr.next());
                    
                    Type type = getChildType();
                    type.write(typeEl, context);
                }
            }
            
            context.put( getKey(), null );
        }
        catch (OgnlException e)
        {
            throw new XFireFault("Couldn't process message.", e, XFireFault.SENDER);
        }
	}

    /**
     * Semantics are still under development, but here is how it works
     * currently:
     * <p>
     * An ArrayList is built for the elements of the array.
     * The number of elements in the array are put in as "key.length". The current
     * index of the array is put in as "key.index". Each child is put in the 
     * array via the OGNL expression (<code>getOgnl()</code>).  Finally,
     * the array is put in the context as "key".  (Where key in all of this
     * refers to the key you specify.)
     * 
     * @see org.codehaus.xfire.aegis.type.Type#read(org.dom4j.Element, java.util.Map)
     */
    public void read( Element element, Map context )
        throws XFireFault 
    {
        read( element, 0, context );
    }
    
    /**
     * @see org.codehaus.xfire.aegis.type.Type#read(org.dom4j.Element, int, java.util.Map)
     */
    public void read(Element root, int occurrence, Map context) throws XFireFault
    {
        List elements = root.elements( getQName() );
        if ( elements.size() <= occurrence )
            return;
        
        Element element = (Element) elements.get(occurrence);

        int size = element.elements().size();
        
        List array = new ArrayList(size);
        context.put( getKey() + ".length", new Integer(size) );

        for ( int i = 0; i < size; i++ )
        {
            context.put( getKey() + ".index", new Integer(i) );

            getChildType().read(element, i, context);
            
            try
            {
                Object value = Ognl.getValue(getOgnl(), context, (Object) null);
                array.add(value);
            }
            catch (OgnlException e)
            {
                throw new XFireFault("Couldn't process message.", e, XFireFault.SENDER);
            }
            
            i++;
        }
        
        if ( array.size() > 0 )
        {
            context.put( getKey(), array );
            
            if ( getPostOgnl() != null )
            {
                try
                {
                    Ognl.getValue( getPostOgnl(), context, array );
                }
                catch (OgnlException e)
                {
                    throw new XFireFault( "Couldn't process message.", e, XFireFault.SENDER );
                }
            }
        }
	}

    public boolean isComplex()
    {
        return true;
    }
    
    public Set getDependencies()
    {
        Set deps = new HashSet();
        
        deps.add( getChildType() );
        
        return deps;
    }
    
    public Type getChildType()
    {
        return childType;
    }
    
    public void setChildType(Type childType)
    {
        this.childType = childType;
    }
    
	public String getKey()
	{
		return key;
	}

	public void setKey(String key)
	{
		this.key = key;
    }

    public String getPostOgnl()
    {
        return postOgnl;
    }
    
    public void setPostOgnl(String postOgnl)
    {
        this.postOgnl = postOgnl;
    }
    
	/**
	 * @see org.codehaus.xfire.aegis.type.Type#writeSchema(org.dom4j.Element)
	 */
	public void writeSchema(Element root)
	{
        Namespace xsdNs = root.getNamespaceForURI(SOAPConstants.XSD);
        Namespace arrayNs = root.getNamespaceForURI( getSchemaType().getNamespaceURI() );
        
        org.dom4j.QName elementQ = new org.dom4j.QName( "element", xsdNs );
        Element concreteEl = root.addElement( elementQ );
        concreteEl.addAttribute("name", getName());
        //concreteEl.addAttribute("type", arrayNs.getPrefix() + ":" + getSchemaType().getName());
        
        org.dom4j.QName complexQ = new org.dom4j.QName("complexType", xsdNs);

		Element complex = concreteEl.addElement(complexQ);
        //complex.addAttribute("name", getName());
        
		org.dom4j.QName seqQ = new org.dom4j.QName("sequence", xsdNs);
		Element seq = complex.addElement(seqQ);

        // Write out the array type.
		Type type = getChildType();
        
        Element element = seq.addElement(elementQ);
        Namespace typeNS = root.getNamespaceForURI( type.getSchemaType().getNamespaceURI() );
        
        if ( type.isComplex() )
        {
            element.addAttribute("ref", typeNS.getPrefix() + ":" + type.getSchemaType().getName());
        }
        else
        {
            element.addAttribute("name", type.getName());
            element.addAttribute("type", typeNS.getPrefix() + ":" + type.getSchemaType().getName());
            element.addAttribute("nillable", "true");
        }
		
		// TODO: Add config support for nillable
		element.addAttribute("minOccurs", "0");
        element.addAttribute("maxOccurs", "unbounded");
	}

	/**
	 * @see org.codehaus.xfire.aegis.type.Type#configure(org.dom4j.Element)
	 */
	public void configure(Element configuration,
                          AegisService service,
                          TypeRegistry reg)
	{
		setName(configuration.attributeValue("name"));
        setOgnl(configuration.attributeValue("ognl"));
        setPostOgnl(configuration.attributeValue("postOgnl"));
        setKey(configuration.attributeValue("key"));
        setDocumentation(configuration.getTextTrim());
        
        setQName(QName.get(getName(), service.getDefaultNamespace()));

		Attribute xsdAt = configuration.attribute("schemaType");
		if (xsdAt != null)
		{
			setSchemaType(null);
		}
		else
		{
			Namespace ns = NamespaceHelper.getNamespace(configuration, service
					.getDefaultNamespace());
			setSchemaType(new QName(getName(), ns));
		}

		if ( configuration.elements().size() > 1 )
        {
            throw new XFireRuntimeException("Only one array child type allowed.");
        }
        
		Element childTypeEl = (Element) configuration.elements().get(0);
		QName typeQ = QName.get(childTypeEl.getName(), service.getSoapVersion());

		Type type = reg.createType(typeQ);
		type.configure(childTypeEl, service, reg);

		setChildType(type);
	}

}