package org.codehaus.xfire.aegis.type;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import ognl.Ognl;
import ognl.OgnlException;

import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.fault.XFireFault;
import org.dom4j.Element;
import org.dom4j.QName;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse </a>
 */
public class FlatArrayType
	extends Type
{
	private String key;
    
    private String arrayClass;
    
    private Type childType;
    
    /**
     * Uses an OGNL expression to find an array.  It then puts each
     * array object in as the key (see <code>getKey()</code> for retrieval
     * when write()'ing in the child type.
     * 
     * @see org.codehaus.xfire.aegis.type.Type#write(org.dom4j.Element, java.util.Map)
     */
	public void write(Element element, Map context) throws XFireFault
	{
		try
        {
            Object value =  Ognl.getValue(getOgnl(), context, (Object) null);

            if ( value == null )
            {
            	return;
            }
            
            if ( value instanceof Object[] )
            {
                Object[] children = (Object[]) value;
                
                for (int i = 0; i < children.length; i++)
                {
                    context.put( getKey(), children[i] );
                    
                    Type type = getChildType();
                    type.write(element, context);
                }
            }
            else if ( value instanceof Collection )
            {
                Collection children = (Collection) value;
                for ( Iterator itr = children.iterator(); itr.hasNext(); )
                {
                    context.put( getKey(), itr.next());
                    
                    Type type = getChildType();
                    type.write(element, context);
                }
            }
            
            context.put( getKey(), null );
        }
        catch (OgnlException e)
        {
            throw new XFireFault("Couldn't process message.", e, XFireFault.SENDER);
        }
	}

    public void read( Element element, Map context )
        throws XFireFault 
    {
        read( element, 0, context );
    }

    
	public void read(Element element, int occurrence, Map context) throws XFireFault
	{
        List array = new ArrayList();
        List elements = element.elements();
        
        context.put( getKey() + ".length", new Integer(elements.size()) );
        
        int i = 0;
        for (Iterator itr = elements.iterator(); itr.hasNext();)
        {
            context.put( getKey() + ".index", new Integer(i) );
            Element arrayEl = (Element) itr.next();
            
            getChildType().read(arrayEl, context);
            
            try
            {
                Object value = Ognl.getValue(getOgnl(), context, (Object) null);

                array.add(value);
            }
            catch (OgnlException e)
            {
                throw new XFireFault("Couldn't process message.", e, XFireFault.SENDER);
            }
            
            i++;
        }
        
        context.put( getKey(), array );
	}

    public boolean isComplex()
    {
        return getChildType().isComplex();
    }
    
    public Set getDependencies()
    {
        return getChildType().getDependencies();
    }
    
    public Type getChildType()
    {
        return childType;
    }
    
    public void setChildType(Type childType)
    {
        this.childType = childType;
    }
    
	public String getKey()
	{
		return key;
	}

	public void setKey(String key)
	{
		this.key = key;
    }
    
	/**
	 * @see org.codehaus.xfire.aegis.type.Type#writeSchema(org.dom4j.Element)
	 */
	public void writeSchema(Element root)
	{
        getChildType().writeSchema(root);
	}

    
	public String getMaxOccurs()
	{
		return getChildType().getMaxOccurs();
	}
    
	public String getMinOccurs()
	{
		return getChildType().getMinOccurs();
	}
    
	public String getName()
	{
		return getChildType().getName();
	}
    
	public QName getQName()
	{
		return getChildType().getQName();
	}
    
    
	public QName getSchemaType()
	{
		return getChildType().getSchemaType();
	}
    
	/**
	 * @see org.codehaus.xfire.aegis.type.Type#configure(org.dom4j.Element)
	 */
	public void configure(Element configuration,
            		      AegisService service,
						  TypeRegistry reg)
	{
		setOgnl(configuration.attributeValue("ognl"));
		setKey(configuration.attributeValue("key"));
        setDocumentation( configuration.getTextTrim() );

		if ( configuration.elements().size() > 1 )
        {
            throw new XFireRuntimeException("Only one array child type allowed.");
        }
        
		Element childTypeEl = (Element) configuration.elements().get(0);
		QName typeQ = QName.get(childTypeEl.getName(), service.getSoapVersion());

		Type type = reg.createType(typeQ);
		type.configure(childTypeEl, service, reg);

		setChildType(type);
	}

}