package org.codehaus.xfire.aegis.type;

import java.util.List;
import java.util.Map;

import ognl.NoSuchPropertyException;
import ognl.Ognl;
import ognl.OgnlException;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.fault.XFireFault;
import org.dom4j.Attribute;
import org.dom4j.Element;
import org.dom4j.QName;

/**
 * An Aegis Type.  Something that is read/written.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Aug 19, 2004
 */
public class SimpleType
    extends Type
{
    private String key;
    
    public SimpleType()
    {
    }

    /**
     * Coerce a value to a string (i.e. create a ISO-8601 string 
     * from a Date).
     * 
     * @param value
     * @return
     */
    public String coerceValue(Object value)
    {
        return value.toString();
    }
    
    /**
     * Coerce the string to an object (i.e. create a date from 
     * a string).
     * 
     * @param string
     * @return
     */
    public Object coerceString(String string)
    {
        return string;
    }
    
    public void write( Element element, Map context ) throws XFireFault 
    {
        Element typeEl = element.addElement( getQName() );
		try
		{
			Object value = Ognl.getValue( getOgnl(), context, (Object) null );
            
            if ( value != null )
            {
                typeEl.setText( coerceValue( value ) );
            }
		}
        catch (NoSuchPropertyException e)
        {
        }
		catch (OgnlException e)
		{
            // TODO: should this be a runtime exception? or a different kind of fault?
			throw new XFireFault( "Couldn't create message.", e, XFireFault.SENDER );
		}
        catch (NullPointerException e)
        {
            throw new XFireRuntimeException( "Couldn't create message type: " + getQName(), e );
        }
		
    }

	public void read( Element element, Map context ) throws XFireFault 
    {
        read( element, 0, context );
    }

    /**
     * @see org.codehaus.xfire.aegis.type.Type#read(org.dom4j.Element, int, java.util.Map)
     */
    public void read(Element element, int occurrence, Map context) throws XFireFault
    {
        List elements = element.elements( getQName() );
        if ( elements.size() <= occurrence )
            return;
        
        Element typeEl = (Element) elements.get(occurrence);
        
        if ( typeEl != null )
        {
            try
			{
                if ( key != null )
                {
                    Object value = Ognl.getValue( getOgnl(), context, coerceString( typeEl.getText() ) );
                    
                    context.put( key, value );
                }
                else
                {
                	Ognl.setValue( getOgnl(), context, (Object) null, coerceString( typeEl.getText() ) );
                }
			}
			catch (OgnlException e)
			{
				throw new XFireFault( "Couldn't process message.", e, XFireFault.SENDER );
			}
        }
    }

	/**
	 * @see org.codehaus.xfire.aegis.type.Type#writeSchema(org.dom4j.Element)
	 */
	public void writeSchema(Element element)
	{
        // Do nothing, we're a simple type.
	}

	/**
	 * @see org.codehaus.xfire.aegis.type.Type#configure(org.dom4j.Element)
	 */
	public void configure(Element configuration, AegisService service, TypeRegistry reg)
	{
		setName( configuration.attribute("name").getStringValue() );
        Attribute ognlAttr = configuration.attribute("ognl");
        if ( ognlAttr == null )
            setOgnl( "#this" );
        else
            setOgnl( ognlAttr.getStringValue() );
        
        setQName( QName.get( getName(), service.getDefaultNamespace() ) );
        setKey( configuration.attributeValue("key") );
        setDocumentation( configuration.getTextTrim() );
        
        Attribute xsdAt = configuration.attribute("schemaType");
        if ( xsdAt != null )
        {
            setSchemaType( null );
        }
        else
        {
            setSchemaType( getDefaultSchemaType() );
        }
	}
    
    public QName getDefaultSchemaType()
    {
        return QName.get("string", SOAPConstants.XSD);
    }

	public String getKey()
	{
		return key;
	}
    
	public void setKey(String key)
	{
		this.key = key;
	}
}
