package org.codehaus.xfire.aegis.type;

import java.util.Map;
import java.util.Set;

import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.mapping.TypeRegistry;
import org.codehaus.xfire.fault.XFireFault;
import org.codehaus.xfire.wsdl.WSDLType;
import org.dom4j.Element;
import org.dom4j.QName;


/**
 * An Aegis Type.  Something that is read/written.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Aug 19, 2004
 */
public abstract class Type
    implements WSDLType
{
    private QName qName;
    private QName schemaType;
    private String ognl;
    private String name;
    private String documentation;
    private String minOccurs;
    private String maxOccurs;
    
    public Type()
    {
        minOccurs = "1";
        maxOccurs = "1";
    }
    
    public abstract void write( Element element, Map context ) throws XFireFault;

	public abstract void read( Element element, Map context ) throws XFireFault;
    
    public abstract void read( Element element, int occurrence, Map context ) throws XFireFault;

    public abstract void writeSchema( Element element ); 
    
    public abstract void configure( Element configuration, AegisService service, TypeRegistry reg );

    public boolean isComplex()
    {
        return false;
    }
    
    public Set getDependencies()
    {
    	return null;
    }
    
    public String getOgnl()
    {
        return ognl;
    }
    
    public void setOgnl( String ognl )
    {
        this.ognl = ognl;
    }
    
    public QName getQName()
    {
        return qName;
    }
    
    public void setQName( QName name )
    {
        qName = name;
    }

	public String getName()
	{
		return name;
	}
    
	public void setName(String name)
	{
		this.name = name;
	}
    
	public QName getSchemaType()
	{
		return schemaType;
	}
    
	public void setSchemaType(QName schemaType)
	{
		this.schemaType = schemaType;
	}
    
	public String getDocumentation()
	{
		return documentation;
	}
    
	public void setDocumentation(String documentation)
	{
		this.documentation = documentation;
	}
    
	public String getMaxOccurs()
	{
		return maxOccurs;
	}
    
	public void setMaxOccurs(String maxOccurs)
	{
		this.maxOccurs = maxOccurs;
	}
    
	public String getMinOccurs()
	{
		return minOccurs;
	}
    
	public void setMinOccurs(String minOccurs)
	{
		this.minOccurs = minOccurs;
	}
}
