package org.codehaus.xfire.aegis.wsdl;

import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.wsdl.Binding;
import javax.wsdl.BindingOperation;
import javax.wsdl.Definition;
import javax.wsdl.Input;
import javax.wsdl.Message;
import javax.wsdl.Operation;
import javax.wsdl.Output;
import javax.wsdl.Part;
import javax.wsdl.Port;
import javax.wsdl.PortType;
import javax.wsdl.WSDLException;
import javax.xml.namespace.QName;

import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.aegis.operation.WrappedOperation;
import org.codehaus.xfire.aegis.type.Type;
import org.codehaus.xfire.transport.Transport;
import org.codehaus.xfire.wsdl.AbstractWSDL;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.Node;

/**
 * AegisWSDL
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 */
public class AegisWSDL
    extends AbstractWSDL
	implements WSDL
{   
    private Set transports;
    
    private Map wsdlOps;
    
    public AegisWSDL(AegisService service, Set transports) 
        throws WSDLException
    {
        super( service );
        this.transports = transports;
        wsdlOps = new HashMap();
        
        Document descriptor = service.getServiceDescriptor();
        
        Element userTypes = descriptor.getRootElement().element("types");
        if ( userTypes != null )
        {
        	addUserTypes(userTypes);
        }
        
        PortType portType = createAbstractInterface();
        
        createConcreteInterface(portType);

        writeDocument();
    }


	/**
	 * @param element
	 */
	private void addUserTypes(Element element)
	{
		for (Iterator itr = element.elementIterator(); itr.hasNext();)
        {
			Element typeEl = (Element) itr.next();
            String name = typeEl.attributeValue("name");
            
            int index = name.indexOf(":");
            String prefix = name.substring(0, index);
            String typeName = name.substring(index+1);

            Namespace ns = element.getNamespaceForPrefix(prefix);
            Element newType = createSchemaType(ns.getURI());
            
            // this is a hack so our namespaces in types still work.
            // Is there a better way?
            newType.add(ns);
            
            int size = typeEl.elements().size();
            for (int i = 0; i < size; i++ )
            {
            	Element te = (Element) typeEl.elements().get(0);
                Node n = te.detach();
                newType.add(n);
            }
        }
	}


	/**
     * Create the PortType for this service.
	 * @param service
	 * @return
	 */
    protected PortType createAbstractInterface()
	{
        AegisService service = (AegisService) getService();
		QName portName = new QName( service.getDefaultNamespace(),
                                    service.getName() + "PortType" );
        
        Definition def = getDefinition();
        
        PortType portType = def.createPortType();
        portType.setQName( portName );
        portType.setUndefined(false);
        def.addPortType( portType );

        
        // Create Abstract operations
        for ( Iterator itr = service.getOperations().iterator(); itr.hasNext(); )
        {
            WrappedOperation op = (WrappedOperation) itr.next();
            Message req = getInputMessage(op);
            def.addMessage( req );
            
            Message res = getOutputMessage(op);
            def.addMessage( res );

            String opName = op.getQName().getName();
            javax.wsdl.Operation wsdlOp = createOperation( opName, req, res );
            wsdlOp.setUndefined(false);
            portType.addOperation( wsdlOp );
            
            wsdlOps.put( opName, wsdlOp );
        }
        
        return portType;
	}

    protected void createConcreteInterface(PortType portType)
    {
        AegisService service = (AegisService) getService();
        QName name = new QName( service.getDefaultNamespace(),
                                 service.getName() );
         
        // Create a concrete instance for each transport.
        javax.wsdl.Service wsdlService = getDefinition().createService();
        wsdlService.setQName( name );

        for ( Iterator itr = transports.iterator(); itr.hasNext(); )
        {
            Transport transport = (Transport) itr.next();
            
            Binding transportBinding = transport.createBinding( portType, service );
            
            for ( Iterator oitr = service.getOperations().iterator(); oitr.hasNext(); )
            {
                // todo: move out of the first loop, we'll be creating req/res multiple times otherwise
                WrappedOperation op = (WrappedOperation) oitr.next();
                
                javax.wsdl.Operation wsdlOp = (javax.wsdl.Operation) wsdlOps.get( op.getQName().getName() );
                
                BindingOperation bop = transport.createBindingOperation( portType, wsdlOp, service );
                transportBinding.addBindingOperation( bop );
            }

            Port transportPort = transport.createPort(transportBinding, service);
            
            getDefinition().addBinding( transportBinding );
            wsdlService.addPort( transportPort );
        }
        
        getDefinition().addService( wsdlService );
    }
    
    /**
     * Create the input and response parts for thes messages.
     * @return
     */
	protected Operation createOperation(String opName, Message req, Message res)
	{
        Definition def = getDefinition();
        
		Input input = def.createInput();
        input.setMessage( req );
        input.setName( req.getQName().getLocalPart() );

        Output output = def.createOutput();
        output.setMessage( res );
        output.setName( res.getQName().getLocalPart() );
        
        javax.wsdl.Operation wsdlOp = def.createOperation();
        wsdlOp.setName( opName );
        wsdlOp.setInput( input );
        wsdlOp.setOutput( output );
        
        return wsdlOp;
	}
    
    private Message getOutputMessage(WrappedOperation op)
    {
        // response message
        Message res = getDefinition().createMessage();
        res.setQName( new QName( getService().getDefaultNamespace(),
                                 op.getQName().getName() + "Response") );

        res.setUndefined(false);

        createOutputParts( res, op );
         
        return res;
    }

	private Message getInputMessage(WrappedOperation op)
    {
        Message req = getDefinition().createMessage();
        req.setQName( new QName( getService().getDefaultNamespace(), op.getQName().getName() + "Request") );
        req.setUndefined(false);
        
        createInputParts( req, op );

        return req;
    }

    private QName createResponseSchemaType(WrappedOperation op, Part part)
    {
        String opName = op.getQName().getName() + "Response";
        QName qName = new QName( getService().getDefaultNamespace(), opName );
        
        Element root = createSchemaType( qName.getNamespaceURI() );
        createParameterTypes(op.getResponse(), opName, root);
        
        return qName;
    }

    private QName createRequestSchemaType(WrappedOperation op, Part part)
    {
        String opName = op.getQName().getName();
        QName qName = new QName( getService().getDefaultNamespace(), opName );
        
        Element root = createSchemaType( qName.getNamespaceURI() );
        createParameterTypes(op.getRequest(), opName, root);
        
        return qName;
    }

	/**
     * A helper method to write out the XSD for the requestion and response.
     * 
     * @param op
     * @param opName
     * @param root
     */
    private void createParameterTypes(List operations, String opName, Element root)
    {
        Element element = root.addElement( elementQ );
        element.addAttribute( "name", opName );
        
        org.dom4j.QName complexQ = new org.dom4j.QName("complexType", xsdNs);
        Element complex = element.addElement( complexQ );
        
        if ( operations.size() > 0 )
        {
            Element sequence = createSequence( complex );
    
            for ( Iterator itr = operations.iterator(); itr.hasNext(); )
            {
                Type type = (Type) itr.next();
                
                // Get the namespace from the schema types document.
                Namespace typeNS = getNamespace( type.getSchemaType().getNamespaceURI() );
                
                org.dom4j.QName outElementQ = new org.dom4j.QName("element", xsdNs);
                Element outElement = sequence.addElement( outElementQ );
                if ( type.isComplex() )
                {
                    outElement.addAttribute( "ref", typeNS.getPrefix() + ":" + type.getSchemaType().getName() );
                }
                else
                {
                	outElement.addAttribute("name", type.getName() );
                    outElement.addAttribute("type", typeNS.getPrefix() + ":" + type.getSchemaType().getName() );
                }
                outElement.addAttribute("minOccurs", type.getMinOccurs());
                outElement.addAttribute("maxOccurs", type.getMaxOccurs());
                
                addDependency( type );
            }
        }
    }


    private Element createSequence( Element complex )
    {
        org.dom4j.QName sequenceQ = new org.dom4j.QName("sequence", xsdNs);
        return complex.addElement( sequenceQ );
    }

    /**
     * @see org.codehaus.xfire.java.wsdl.AbstractWSDL#createInputParts(javax.wsdl.Message, org.codehaus.xfire.java.Operation)
     */
    protected void createInputParts( Message req, WrappedOperation op )
    {
        Part part = getDefinition().createPart();
            
        QName typeQName = createRequestSchemaType(op, part);
        
        part.setName( "parameters" );
        part.setElementName( typeQName );
        
        req.addPart( part );
    }
    
    /**
     * Create the output parts for this message/operation. This involves
     * creating an part with the name "parameters" which refers to
     * a type that we create for the request/response.
     * 
     * @param req
     * @param op
     */
    protected void createOutputParts( Message req, WrappedOperation op )
    {
        // response message part
        Part part = getDefinition().createPart();

        // Document style service
        QName typeQName = createResponseSchemaType(op, part);
        part.setElementName( typeQName );
        part.setName( "parameters" );
        
        req.addPart( part );
    }

}
