package org.codehaus.xfire.aegis.wsdl;

import java.util.Set;

import javax.wsdl.WSDLException;

import org.codehaus.plexus.component.repository.exception.ComponentLookupException;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.ServiceLocator;
import org.codehaus.plexus.personality.plexus.lifecycle.phase.Serviceable;
import org.codehaus.xfire.AbstractXFireComponent;
import org.codehaus.xfire.XFireRuntimeException;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.service.Service;
import org.codehaus.xfire.service.ServiceRegistry;
import org.codehaus.xfire.transport.TransportManager;
import org.codehaus.xfire.wsdl.WSDL;
import org.codehaus.xfire.wsdl.WSDLBuilder;

/**
 * Create a WSDL document for an AegisService.
 * 
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Apr 7, 2004
 */
public class AegisWSDLBuilder
	extends AbstractXFireComponent
	implements WSDLBuilder, Serviceable
{
    private ServiceLocator manager;
    
	public WSDL createWSDL( Service s )
        throws WSDLException
	{
        AegisService service = (AegisService) s;
        
	    TransportManager manager;
        try
        {
            manager = (TransportManager) getServiceLocator().lookup( TransportManager.ROLE );
        }
        catch (ComponentLookupException e)
        {
            throw new XFireRuntimeException( "Couldn't find TransportManager.", e );
        }
        
        Set transports = manager.getTransports( service.getName() );
	    
		if ( service.getStyle().equals("wrapped")
             &&
             service.getUse().equals("literal") )
        {
            return new AegisWSDL( service, transports );
        }
        /*else if ( service.getStyle().equals("document")
                  &&
                  service.getUse().equals("literal") )
        {
            return new DocumentWSDL( service, transports );
        }
        else if ( service.getStyle().equals("rpc")
                &&
                service.getUse().equals("literal") )
        {
            return new RPCLiteralWSDL( service, transports );
        }*/
        else
        {
        	throw new UnsupportedOperationException( "Service style/use not supported." );
        }
	}
    
    protected ServiceRegistry getServiceRegistry()
    {
        ServiceRegistry registry = null;
        
        try
        {
            registry = (ServiceRegistry) getServiceLocator().lookup( ServiceRegistry.ROLE );
        }
        catch (ComponentLookupException e)
        {
            throw new RuntimeException( "Couldn't find the ServiceRegistry!", e );
        }
        
        return registry;
    }
    
    public void service( ServiceLocator manager )
    {
        this.manager = manager;
    }
    
    /**
     * @return Returns the service manager.
     */
    protected ServiceLocator getServiceLocator()
    {
        return manager;
    }
}
