package org.codehaus.xfire.aegis;

import java.util.HashMap;
import org.codehaus.xfire.aegis.operation.WrappedOperation;
import org.codehaus.xfire.aegis.type.BeanType;
import org.codehaus.xfire.plexus.PlexusXFireTest;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class AegisServiceProviderTest
    extends PlexusXFireTest
{
    public void testServiceRegister() throws Exception
    {
        AegisService hello = (AegisService) lookup( AegisService.ROLE, "Book" );
        
        assertEquals("http://xfire.codehaus.org/book", hello.getDefaultNamespace());
        assertEquals("http://www.w3.org/2003/05/soap-envelope", hello.getSoapVersion());
        
        assertNotNull( hello.getOperations() );
        assertEquals( 2, hello.getOperations().size() );
        
        WrappedOperation op = (WrappedOperation) hello.getOperations().iterator().next();
        assertNotNull( op.getRequest() );
        assertEquals( 1, op.getRequest().size() );
        assertEquals( "Finds a book.", op.getDocumentation() );
        
        assertNotNull( op.getResponse() );
        assertEquals( 1, op.getResponse().size() );
        BeanType bt = (BeanType) op.getResponse().get(0);
        assertNotNull( bt.getChildren() );
        assertEquals( 3, bt.getChildren().size() );
        
        Document doc = DocumentHelper.createDocument();
        Element root = doc.addElement("root");
        
        HashMap map = new HashMap();
        map.put("author", "Dan Diephouse");
        map.put("title", "Boring Biography");
        
        bt.write(root, map);
        
        addNamespace("book", "http://xfire.codehaus.org/book");
        assertValid("/root/book:Book/book:author[text()='Dan Diephouse']", root);
        assertValid("/root/book:Book/book:title[text()='Boring Biography']", root);
    }
    
    public void testFindBook() throws Exception
    {
        WSDL wsdl = getWSDL("Book");

        Document wsdlDoc = getWSDLDocument("Book");
        
        Document response = invokeService("Book", "/org/codehaus/xfire/aegis/book/FindBook.xml");
        
        addNamespace("book", "http://xfire.codehaus.org/book");
        assertValid("/soap12:Envelope/soap12:Body/book:FindBookResponse/book:Book", response);
        assertValid("//book:Book/book:author[text()='Dan Diephouse']", response);
        assertValid("//book:Book/book:title[text()='Boring Biography']", response);
        assertValid("//book:Book/book:isbn[text()='0123456789']", response);
   }

    public void testSearchBooks() throws Exception
    {
        Document response = invokeService("Book", "/org/codehaus/xfire/aegis/book/SearchBooks.xml");

        addNamespace("book", "http://xfire.codehaus.org/book");
        assertValid("/soap12:Envelope/soap12:Body/book:SearchResponse/book:BookArray", response);
        assertValid("//book:BookArray/book:Book/book:author[text()='Dan Diephouse']", response);
        assertValid("//book:BookArray/book:Book/book:title[text()='Boring Biography']", response);
        assertValid("//book:BookArray/book:Book/book:isbn[text()='0123456789']", response);
    }
    
    public void testPrimitives() throws Exception
    {
        Document response = invokeService("Test", "/org/codehaus/xfire/aegis/test/PrimitiveTest.xml");
        
        addNamespace("t", "urn:Test");
        assertValid("/soap12:Envelope/soap12:Body/t:PrimitiveTestResponse/t:string[text()='bleh']", response);
        assertValid("/soap12:Envelope/soap12:Body/t:PrimitiveTestResponse/t:long[text()='10']", response);
        assertValid("/soap12:Envelope/soap12:Body/t:PrimitiveTestResponse/t:int[text()='10']", response);
    }
    
    public void testEmptyRequest() throws Exception
    {
        Document response = invokeService("Test", "/org/codehaus/xfire/aegis/test/EmptyRequest.xml");
        
        addNamespace("book", "urn:Test");
        assertValid("/soap12:Envelope/soap12:Body/book:EmptyRequestResponse", response);
        assertValid("//book:EmptyRequestResponse/book:test[text()='test']", response);
    }

    public void testEmptyResponse() throws Exception
    {
        Document response = invokeService("Test", "/org/codehaus/xfire/aegis/test/EmptyResponse.xml");
        
        addNamespace("book", "urn:Test");
        assertValid("/soap12:Envelope/soap12:Body/book:EmptyResponseResponse", response);
    }
    
    public void testHeaders() throws Exception
    {
        Document response = invokeService("Test", "/org/codehaus/xfire/aegis/test/HeaderTest.xml");
        
        addNamespace("t", "urn:Test");
        assertValid("/soap12:Envelope/soap12:Header/t:test[text()='bleh']", response);
        assertValid("/soap12:Envelope/soap12:Body/t:HeaderTestResponse", response);
    }
    
    public void testFlatArray() throws Exception
    {
        Document response = invokeService("Test", "/org/codehaus/xfire/aegis/test/GetFlatArray.xml");
        
        addNamespace("t", "urn:Test");
        assertValid("/soap12:Envelope/soap12:Body/t:GetFlatArrayResponse/t:string[text()='hi']", response);
        assertValid("/soap12:Envelope/soap12:Body/t:GetFlatArrayResponse/t:string[text()='ho']", response);
    }
    
    /*public void testCustomTypes() throws Exception
    {
        WSDL wsdl = getWSDL("Test");
        Document response = wsdl.getDocument();
        
        wsdl.write(System.out);
        addNamespace("wsdl", WSDL.WSDL11_NS);
        addNamespace("xsd", SOAPConstants.XSD);
        assertValid("//xsd:schema[@targetNamespace='urn:Test']/xsd:element[@name='State'][@type='tns:State']", response);
        assertValid("//xsd:schema[@targetNamespace='urn:Test']/xsd:complexType[@name='State']", response);
    }*/
}
