package org.codehaus.xfire.aegis.type;

import java.util.Date;
import java.util.HashMap;
import java.util.Map;

import junit.framework.TestCase;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.dom4j.Namespace;
import org.dom4j.QName;
import org.dom4j.io.SAXReader;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Aug 19, 2004
 */
public class TypeTest
    extends TestCase
{
    public void testSimpleType()
        throws Exception
    {
        SAXReader reader = new SAXReader();
        Document doc = DocumentHelper.createDocument();
        
        Namespace ns = DocumentHelper.createNamespace("test", "urn:test");
        QName qname = DocumentHelper.createQName("bleh", ns);
        QName rootQname = DocumentHelper.createQName("root", ns);
        
        Element root = doc.addElement(rootQname);
        
        Map map = new HashMap();
        SimpleBean bean = new SimpleBean();
        map.put("bean", bean);
        
        bean.setBleh("bleh");
        
        Type type = new SimpleType();
        type.setOgnl("#bean.bleh");
        type.setQName(qname);
        
        type.write( root, map );
        
        Element typeEl = root.element( qname );
        assertNotNull(typeEl);
        assertEquals("bleh", typeEl.getText());
        
        map = new HashMap();
        bean = new SimpleBean();
        map.put("bean", bean);
        
        type.read( root, map );
        
        assertEquals( "bleh", bean.getBleh() );
    }
    
    public void testDateType()
        throws Exception
    {
        SAXReader reader = new SAXReader();
        Document doc = DocumentHelper.createDocument();
        
        Namespace ns = DocumentHelper.createNamespace("test", "urn:test");
        QName qname = DocumentHelper.createQName("bleh", ns);
        QName rootQname = DocumentHelper.createQName("root", ns);
        
        Element root = doc.addElement(rootQname);
        
        Map map = new HashMap();
        SimpleBean bean = new SimpleBean();
        map.put("bean", bean);
        
        bean.setDate( new Date() );
        
        Type type = new DateType();
        type.setOgnl("#bean.date");
        type.setQName(qname);
        
        type.write( root, map );
        
        Element typeEl = root.element( qname );
        assertNotNull(typeEl);
        assertNotNull(typeEl.getText());
        System.out.println(typeEl.getText());
        
        map = new HashMap();
        bean = new SimpleBean();
        map.put("bean", bean);
        
        type.read( root, map );
        
        assertNotNull( bean.getDate() );
    }
    
    public void testIntegerType()
        throws Exception
    {
        SAXReader reader = new SAXReader();
        Document doc = DocumentHelper.createDocument();
        
        Namespace ns = DocumentHelper.createNamespace("test", "urn:test");
        QName qname = DocumentHelper.createQName("bleh", ns);
        QName rootQname = DocumentHelper.createQName("root", ns);
        
        Element root = doc.addElement(rootQname);
        
        Map map = new HashMap();
        SimpleBean bean = new SimpleBean();
        map.put("bean", bean);
        
        bean.setInteger(10);
        
        Type type = new IntegerType();
        type.setOgnl("#bean.integer");
        type.setQName(qname);
        
        type.write( root, map );
        
        Element typeEl = root.element( qname );
        assertNotNull(typeEl);
        assertEquals("10", typeEl.getText());
        
        map = new HashMap();
        bean = new SimpleBean();
        map.put("bean", bean);
        
        type.read( root, map );
        
        assertEquals( 10, bean.getInteger() );
    }
    
    public void testBeanType()
        throws Exception
    {
        SAXReader reader = new SAXReader();
        Document doc = DocumentHelper.createDocument();
        
        Namespace ns = DocumentHelper.createNamespace("test", "urn:test");
        QName beanQ = DocumentHelper.createQName("bean", ns);
        QName blehQ = DocumentHelper.createQName("bleh", ns);
        QName rootQ = DocumentHelper.createQName("root", ns);
        
        Element root = doc.addElement(rootQ);
        
        Map map = new HashMap();
        SimpleBean bean = new SimpleBean();
        map.put("bean", bean);
        
        bean.setBleh("bleh");
        
        BeanType type = new BeanType();
        type.setQName(beanQ);
        type.setOgnl("new org.codehaus.xfire.aegis.type.SimpleBean()");
        type.setKey("bean");
        
        SimpleType child = new SimpleType();
        child.setOgnl("#bean.bleh");
        child.setQName(blehQ);
        type.addChild(child);
        
        type.write( root, map );
        
        Element beanEl = root.element( beanQ );
        assertNotNull(beanEl);
        assertEquals("", beanEl.getText());
        
        Element blehEl = beanEl.element( blehQ );
        assertNotNull(blehEl);
        assertEquals("bleh", blehEl.getText());
        
        map = new HashMap();

        type.read( root, map );
        
        assertEquals( "bleh", bean.getBleh() );
    }
}
