package org.codehaus.xfire.aegis.wsdl;

import org.codehaus.xfire.SOAPConstants;
import org.codehaus.xfire.aegis.AegisService;
import org.codehaus.xfire.plexus.PlexusXFireTest;
import org.codehaus.xfire.wsdl.WSDL;
import org.dom4j.Document;
import org.dom4j.io.OutputFormat;
import org.dom4j.io.XMLWriter;

/**
 * @author <a href="mailto:dan@envoisolutions.com">Dan Diephouse</a>
 * @since Feb 21, 2004
 */
public class AegisWSDLTest
    extends PlexusXFireTest
{
    public void testWSDL() throws Exception
    {
        AegisService hello = (AegisService) lookup( AegisService.ROLE, "Book" );

        AegisWSDL wsdl = (AegisWSDL) hello.getWSDL();
        assertNotNull( wsdl );
        
        Document doc = wsdl.getDocument();
        assertNotNull( doc );
        
        XMLWriter writer = new XMLWriter( OutputFormat.createPrettyPrint() );
        writer.setOutputStream( System.out );
        writer.write( wsdl.getDocument() );
        
        addNamespace("book", "http://xfire.codehaus.org/book");
        addNamespace( "wsdl", WSDL.WSDL11_NS );
        addNamespace( "xsd", SOAPConstants.XSD );

        // Types
        assertValid( "/wsdl:definitions/wsdl:types", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema", doc );
        assertValid( "/wsdl:definitions/wsdl:types/xsd:schema[@targetNamespace='http://xfire.codehaus.org/book']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='FindBook']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='FindBookResponse']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='FindBook']/xsd:complexType/xsd:sequence/xsd:element[@name='isbn'][@type=\"xsd:string\"]", doc );
        assertValid( "//xsd:schema/xsd:element[@name='Search']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='SearchResponse']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='Search']/xsd:complexType/xsd:sequence/xsd:element[@name='keyword'][@type=\"xsd:string\"]", doc );
        assertValid( "//xsd:schema/xsd:element[@name='Book']", doc );
        assertValid( "//xsd:schema/xsd:element[@name='BookArray']", doc );
        assertValid( "//xsd:element[@name='Book']/xsd:complexType/xsd:sequence/xsd:element[@name='author'][@type='xsd:string']", doc );
        assertValid( "//xsd:element[@name='Book']/xsd:complexType/xsd:sequence/xsd:element[@name='title'][@type='xsd:string']", doc );
        assertValid( "//xsd:element[@name='Book']/xsd:complexType/xsd:sequence/xsd:element[@name='isbn'][@type='xsd:string']", doc );
        assertValid( "//xsd:element[@name='BookArray']/xsd:complexType/xsd:sequence/xsd:element[@ref='tns:Book']", doc );
        
        // messages
        assertValid( "//wsdl:message[@name='FindBookRequest']/wsdl:part[@element='tns:FindBook']", doc );
        assertValid( "//wsdl:message[@name='FindBookResponse']/wsdl:part[@element='tns:FindBookResponse']", doc );
        assertValid( "//wsdl:message[@name='SearchRequest']/wsdl:part[@element='tns:Search']", doc );
        assertValid( "//wsdl:message[@name='SearchResponse']/wsdl:part[@element='tns:SearchResponse']", doc );
        
        // Abstract interface
        assertValid( "//wsdl:portType[@name='BookPortType']", doc );
        assertValid( "//wsdl:portType[@name='BookPortType']/wsdl:operation[@name='FindBook']", doc );
        assertValid( "//wsdl:portType[@name='BookPortType']/wsdl:operation[@name='Search']", doc );
        
    }
}
